@@ -1,9 +1,9 @@
-# $Id: BR.pm 90 2005-08-27 10:58:31Z chronos $
+# $Id: BR.pm 158 2006-02-04 19:12:54Z chronos $
 package BBCode::Tag::BR;
 use base qw(BBCode::Tag);
 use strict;
 use warnings;
-our $VERSION = '0.01';
+our $VERSION = '0.30';
 
 sub Class($):method {
 	return qw(TEXT INLINE);
@@ -17,4 +17,8 @@ sub toHTML($):method {
 	return "<br/>";
 }
 
+sub toText($):method {
+	return "\n";
+}
+
 1;
