@@ -1,9 +1,10 @@
-# $Id: HIDDEN.pm 90 2005-08-27 10:58:31Z chronos $
+# $Id: HIDDEN.pm 158 2006-02-04 19:12:54Z chronos $
 package BBCode::Tag::HIDDEN;
 use base qw(BBCode::Tag::Inline);
+use BBCode::Util qw(multilineText);
 use strict;
 use warnings;
-our $VERSION = '0.01';
+our $VERSION = '0.30';
 
 sub BodyPermitted($):method {
 	return 1;
@@ -19,7 +20,7 @@ sub toHTML($):method {
 		$ret .= $_->toHTML;
 	}
 	$ret .= '</span>';
-	return $ret;
+	return multilineText $ret;
 }
 
 1;
