@@ -1,9 +1,9 @@
-# $Id: LI.pm 117 2006-01-17 14:36:56Z chronos $
+# $Id: LI.pm 158 2006-02-04 19:12:54Z chronos $
 package BBCode::Tag::LI;
 use base qw(BBCode::Tag::Simple BBCode::Tag);
 use strict;
 use warnings;
-our $VERSION = '0.23';
+our $VERSION = '0.30';
 
 sub BodyPermitted($):method {
 	return 1;
@@ -11,7 +11,7 @@ sub BodyPermitted($):method {
 
 sub BodyTags($):method {
 	# Despite previous reports to the contrary, :BLOCK is fine here
-	return qw(:LIST :BLOCK :INLINE);
+	return qw(:BLOCK :INLINE);
 }
 
 sub toHTML($):method {
