@@ -1,9 +1,10 @@
-# $Id: U.pm 90 2005-08-27 10:58:31Z chronos $
+# $Id: U.pm 158 2006-02-04 19:12:54Z chronos $
 package BBCode::Tag::U;
 use base qw(BBCode::Tag::Inline);
+use BBCode::Util qw(multilineText);
 use strict;
 use warnings;
-our $VERSION = '0.01';
+our $VERSION = '0.30';
 
 sub BodyPermitted($):method {
 	return 1;
@@ -15,11 +16,13 @@ sub toHTML($):method {
 	my $css = $this->parser->css_direct_styles ? qq( style="text-decoration: underline") : "";
 
 	my $ret = qq(<span class="${pfx}u"$css>);
-	foreach($this->body) {
-		$ret .= $_->toHTML;
-	}
+	$ret .= $this->bodyHTML;
 	$ret .= '</span>';
-	return $ret;
+	return multilineText $ret;
+}
+
+sub toText($):method {
+	return multilineText '_'.shift->bodyText().'_';
 }
 
 1;
