@@ -1,10 +1,10 @@
-# $Id: xIMG.pm 90 2005-08-27 10:58:31Z chronos $
+# $Id: xIMG.pm 158 2006-02-04 19:12:54Z chronos $
 package BBCode::Tag::xIMG;
 use base qw(BBCode::Tag::Inline);
 use BBCode::Util qw(:parse :encode :text);
 use strict;
 use warnings;
-our $VERSION = '0.01';
+our $VERSION = '0.30';
 
 sub Tag($):method {
 	return 'IMG';
@@ -31,16 +31,4 @@ sub replace($):method {
 	}
 }
 
-sub toBBCode($):method {
-	return shift->replace->toBBCode;
-}
-
-sub toHTML($):method {
-	return shift->replace->toHTML;
-}
-
-sub toLinkList($):method {
-	return shift->replace->toLinkList;
-}
-
 1;
