@@ -1,11 +1,10 @@
-# $Id: xURL.pm 90 2005-08-27 10:58:31Z chronos $
+# $Id: xURL.pm 158 2006-02-04 19:12:54Z chronos $
 package BBCode::Tag::xURL;
 use base qw(BBCode::Tag);
 use BBCode::Util qw(:parse :encode :text);
-use HTML::Entities ();
 use strict;
 use warnings;
-our $VERSION = '0.01';
+our $VERSION = '0.30';
 
 sub Tag($):method {
 	return 'URL';
@@ -39,16 +38,4 @@ sub replace($):method {
 	}
 }
 
-sub toBBCode($):method {
-	return shift->replace->toBBCode;
-}
-
-sub toHTML($):method {
-	return shift->replace->toHTML;
-}
-
-sub toLinkList($):method {
-	return shift->replace->toLinkList;
-}
-
 1;
